color_square_canvas = love.graphics.newCanvas(256, 256)
COLOR_SQUARE_X_OFFSET = 100
COLOR_SQUARE_Y_OFFSET = 100

hue_canvas = love.graphics.newCanvas(32, 256)
HUE_X_OFFSET = 400
HUE_Y_OFFSET = 100

hue = 0
saturation = 0
value = 0

function love.load()
    create_color_square_canvas()
    create_hue_canvas()
end

function love.draw()
    love.graphics.draw(color_square_canvas, COLOR_SQUARE_X_OFFSET, COLOR_SQUARE_Y_OFFSET)
    love.graphics.draw(hue_canvas, HUE_X_OFFSET, HUE_Y_OFFSET)

    love.graphics.setColor(0, 0, 0)
    love.graphics.circle("line", COLOR_SQUARE_X_OFFSET + saturation, COLOR_SQUARE_Y_OFFSET + value, 10)

    love.graphics.setColor(1, 1, 1)
    love.graphics.circle("line", COLOR_SQUARE_X_OFFSET + saturation, COLOR_SQUARE_Y_OFFSET + value, 9)
    love.graphics.line(HUE_X_OFFSET-5, HUE_Y_OFFSET + hue, HUE_X_OFFSET+32+5, HUE_Y_OFFSET + hue)

    love.graphics.print("hue", 405, 360)
    love.graphics.print("saturation", 200, 360)
    love.graphics.print("value", 60, 220)

    love.graphics.print("hue: " .. hue, 500, 100)
    love.graphics.print("saturation: " .. saturation, 500, 120)
    love.graphics.print("value: " .. value, 500, 140)

    r, g, b = HSV(hue, saturation, value)
    red = math.floor(r)
    green = math.floor(g)
    blue = math.floor(b)
    love.graphics.print("red: " .. red, 500, 200)
    love.graphics.print("green: " .. green, 500, 220)
    love.graphics.print("blue: " .. blue, 500, 240)

end

function love.mousepressed(x, y)
    if (x >= COLOR_SQUARE_X_OFFSET and x <= COLOR_SQUARE_X_OFFSET + 256) and (y >= COLOR_SQUARE_Y_OFFSET and y <= COLOR_SQUARE_Y_OFFSET + 256) then
        saturation = x - COLOR_SQUARE_X_OFFSET
        value = y - COLOR_SQUARE_Y_OFFSET
    end

    if (x >= HUE_X_OFFSET and x <= HUE_X_OFFSET + 32) and (y >= HUE_Y_OFFSET and y <= HUE_Y_OFFSET + 256) then
        hue = y - HUE_Y_OFFSET
        create_color_square_canvas()
    end
end

function love.keypressed(key)
    if key == "escape" then
        love.event.quit()
    end
end

function create_hue_canvas()
    for hue=0,255 do
        r, g, b = HSV(hue, 255, 255)
        hue_canvas:renderTo(function()
            love.graphics.setColor(r/256, g/256, b/256);
            love.graphics.line(0, hue, 31, hue);
        end);
    end
end

function create_color_square_canvas()
    for saturation=0,255 do
        for value=0,255 do
            r, g, b = HSV(hue, saturation, value)
            color_square_canvas:renderTo(function()
                love.graphics.setColor(r/256, g/256, b/256);
                love.graphics.points(saturation, value);
            end);
        end
    end
end

-- source for this function: https://love2d.org/wiki/HSV_color
function HSV(h, s, v)
    if s <= 0 then return v,v,v end
    h, s, v = h/256*6, s/255, v/255
    local c = v*s
    local x = (1-math.abs((h%2)-1))*c
    local m,r,g,b = (v-c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end return (r+m)*255,(g+m)*255,(b+m)*255
end
